 /*
  *  fonulator 3 - foneBRIDGE configuration daemon
  *  (C) 2005-2007 Redfone Communications, LLC.
  *   www.red-fone.com
  *
  * Doubly Linked List Implementation
  * from "Mastering Algorithms with C"
  */

#ifndef DLIST_H
#define DLIST_H

#include <stdlib.h>

/*******************************************************************************
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF, OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE, OR ANY OTHER DEALINGS IN
 * THE SOFTWARE.
 ********************************************************************************/


/*****************************************************************************
 *                                                                            *
 *  Define a structure for doubly-linked list elements.                       *
 *                                                                            *
 *****************************************************************************/

typedef struct DListElmt_
{

  void *data;
  struct DListElmt_ *prev;
  struct DListElmt_ *next;

} DListElmt;

/*****************************************************************************
 *                                                                            *
 *  Define a structure for doubly-linked lists.                               *
 *                                                                            *
 *****************************************************************************/

typedef struct DList_
{

  int size;

  int (*match) (const void *key1, const void *key2);
  void (*destroy) (void *data);

  DListElmt *head;
  DListElmt *tail;

} DList;

/*****************************************************************************
 *                                                                            *
 *  --------------------------- Public Interface ---------------------------  *
 *                                                                            *
 *****************************************************************************/

void dlist_init (DList * list, void (*destroy) (void *data));

void dlist_destroy (DList * list);

int dlist_ins_next (DList * list, DListElmt * element, const void *data);

int dlist_ins_prev (DList * list, DListElmt * element, const void *data);

int dlist_remove (DList * list, DListElmt * element, void **data);

#define dlist_size(list) ((list)->size)

#define dlist_head(list) ((list)->head)

#define dlist_tail(list) ((list)->tail)

#define dlist_is_head(element) ((element)->prev == NULL ? 1 : 0)

#define dlist_is_tail(element) ((element)->next == NULL ? 1 : 0)

#define dlist_data(element) ((element)->data)

#define dlist_next(element) ((element)->next)

#define dlist_prev(element) ((element)->prev)

#endif
